@@ -202,8 +202,16 @@ namespace Boo.Lang.Compiler.Ast
 		
 		public static bool IsTargetOfMethodInvocation(Expression node)
 		{
-			return node.ParentNode.NodeType == NodeType.MethodInvocationExpression &&
-					node == ((MethodInvocationExpression)node.ParentNode).Target;
+			return IsTargetOfGenericMethodInvocation(node) ||
+				(node.ParentNode.NodeType == NodeType.MethodInvocationExpression &&
+					node == ((MethodInvocationExpression)node.ParentNode).Target);
+		}
+
+		public static bool IsTargetOfGenericMethodInvocation(Expression node)
+		{
+            return node.ParentNode.NodeType == NodeType.GenericReferenceExpression && node.ParentNode.ParentNode != null
+                    && node.ParentNode.ParentNode.NodeType == NodeType.MethodInvocationExpression
+                    && node.ParentNode == ((MethodInvocationExpression)node.ParentNode.ParentNode).Target;
 		}
 
 		public static bool IsTargetOfMemberReference(Expression node)
