@@ -984,6 +984,20 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Visit(fs.Iterator);
 			WriteLine(":");
 			WriteBlock(fs.Block);
+			if(fs.OrBlock != null)
+			{
+				WriteIndented();
+				WriteKeyword("or:");
+				WriteLine();
+				WriteBlock(fs.OrBlock);
+			}
+			if(fs.ThenBlock != null)
+			{
+				WriteIndented();
+				WriteKeyword("then:");
+				WriteLine();
+				WriteBlock(fs.ThenBlock);
+			}
 		}
 		
 		override public void OnTryStatement(TryStatement node)
@@ -1097,6 +1111,20 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		override public void OnWhileStatement(WhileStatement node)
 		{
 			WriteConditionalBlock("while", node.Condition, node.Block);
+			if(node.OrBlock != null)
+			{
+				WriteIndented();
+				WriteKeyword("or:");
+				WriteLine();
+				WriteBlock(node.OrBlock);
+			}
+			if(node.ThenBlock != null)
+			{
+				WriteIndented();
+				WriteKeyword("then:");
+				WriteLine();
+				WriteBlock(node.ThenBlock);
+			}
 		}
 
 		override public void OnIfStatement(IfStatement node)
