@@ -339,7 +339,7 @@ namespace Boo.Lang.Compiler
 			return new CompilerError("BCE0057", SafeLexicalInfo(node), name);
 		}
 
-		public static CompilerError ObjectRequired(Node node)
+		public static CompilerError SelfIsNotValidInStaticMember(Node node)
 		{
 			return new CompilerError("BCE0058", SafeLexicalInfo(node));
 		}
@@ -834,6 +834,11 @@ namespace Boo.Lang.Compiler
 		{
 			return new CompilerError("BCE0156", SafeLexicalInfo(node), ev.Name, ev.DeclaringType);
 		}
+		
+		public static CompilerError GenericTypesMustBeConstructedToBeInstantiated(Node node)
+		{
+			return new CompilerError("BCE0157", SafeLexicalInfo(node));
+		}
 
 		public static string ToStringList(System.Collections.IEnumerable names)
 		{
