@@ -508,6 +508,9 @@ namespace Boo.Lang.Compiler.Steps
 
 			//	<block>
 			ws.Block.Add(node.Block);
+			
+			ws.OrBlock = node.OrBlock;
+			ws.ThenBlock = node.ThenBlock;
 
 			body.Add(ws);
 
@@ -616,6 +619,8 @@ namespace Boo.Lang.Compiler.Steps
 			AstAnnotations.MarkUnchecked(assignment);
 			
 			ws.Block.Add(assignment);
+			ws.OrBlock = node.OrBlock;
+			ws.ThenBlock = node.ThenBlock;
 			body.Add(ws);
 			ReplaceCurrentNode(body);
 		}
