@@ -98,8 +98,32 @@ namespace Boo.Lang.Compiler.TypeSystem
 		private IType CreateConcreteCallableType(Node sourceNode, AnonymousCallableType anonymousType)
 		{
 			Module module = TypeSystemServices.GetCompilerGeneratedTypesModule();
-
-			string name = string.Format("___callable{0}", module.Members.Count);
+			
+			TypeMember enclosing = (sourceNode.GetAncestor(NodeType.ClassDefinition) ?? sourceNode.GetAncestor(NodeType.InterfaceDefinition) ?? sourceNode.GetAncestor(NodeType.EnumDefinition) ?? sourceNode.GetAncestor(NodeType.Module)) as TypeMember;
+			string prefix = "";
+			string postfix = "";
+			if(enclosing != null)
+			{
+				prefix += enclosing.Name;
+				enclosing = (sourceNode.GetAncestor(NodeType.Method) 
+						?? sourceNode.GetAncestor(NodeType.Property) 
+						?? sourceNode.GetAncestor(NodeType.Event) 
+						?? sourceNode.GetAncestor(NodeType.Field)) as TypeMember;
+				if(enclosing != null)
+				{
+					prefix += "_" + enclosing.Name;
+				}
+				prefix += "$";
+			}
+			else if (!sourceNode.LexicalInfo.Equals(LexicalInfo.Empty))
+			{
+				prefix += System.IO.Path.GetFileNameWithoutExtension(sourceNode.LexicalInfo.FileName) + "$";
+			}
+			if(!sourceNode.LexicalInfo.Equals(LexicalInfo.Empty))
+			{
+				postfix = "$" + sourceNode.LexicalInfo.Line + "_" + sourceNode.LexicalInfo.Column + postfix;
+			}
+			string name = "__" + prefix + "callable" + module.Members.Count + postfix + "__";
 			ClassDefinition cd = TypeSystemServices.CreateCallableDefinition(name);
 			cd.Modifiers |= TypeMemberModifiers.Public;
 			cd.LexicalInfo = sourceNode.LexicalInfo;
@@ -144,12 +168,15 @@ namespace Boo.Lang.Compiler.TypeSystem
 			Method extension = CodeBuilder.CreateMethod("BeginInvoke", TypeSystemServices.Map(typeof(IAsyncResult)),
 										TypeMemberModifiers.Public | TypeMemberModifiers.Static);
 			extension.Attributes.Add(CodeBuilder.CreateAttribute(Types.BooExtensionAttribute));
-			extension.Attributes.Add(CodeBuilder.CreateAttribute(Types.ClrExtensionAttribute));
+			if (MetadataUtil.HasClrExtensions())
+			{
+				extension.Attributes.Add(CodeBuilder.CreateAttribute(Types.ClrExtensionAttribute));
+			}
 
 			ParameterDeclaration self = CodeBuilder.CreateParameterDeclaration(0, "self", beginInvokeEntity.DeclaringType);
 
 			extension.Parameters.Add(self);
-			CodeBuilder.DeclareParameters(extension, 1, anonymousType.GetSignature().Parameters);
+			CodeBuilder.DeclareParameters(extension, anonymousType.GetSignature().Parameters, 1);
 
 			mie = CodeBuilder.CreateMethodInvocation(
 						CodeBuilder.CreateReference(self),
@@ -219,4 +246,5 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return CodeBuilder.CreateRuntimeMethod("Invoke", signature.ReturnType, signature.Parameters, signature.AcceptVarArgs);
 		}
 	}
-}
\ No newline at end of file
+}
+
