@@ -58,6 +58,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 				declaration.ParentNode as TypeDefinition : _declaringMethod.DeclaringType);
 		}
 
+		public InternalGenericParameter(TypeSystemServices tss, GenericParameterDeclaration declaration, int position)
+				: this(tss, declaration)
+		{
+			_position = position;
+		}
+
 		public int GenericParameterPosition
 		{
 			get 
@@ -276,5 +282,25 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return FullName;
 		}
 
+		override public bool Equals(object rhs)
+		{
+			IGenericParameter p = rhs as IGenericParameter;
+			if (null == p) return false;
+
+			//TODO: >=0.9 : check base type constraints
+			return Name == p.Name //FIXME: should be GenericParameterPosition but crashes on internal g. params(?!!)
+				&& Variance == p.Variance
+				&& MustHaveDefaultConstructor  == p.MustHaveDefaultConstructor
+				&& IsClass == p.IsClass
+				&& IsValueType == p.IsValueType;
+		}
+
+		override public int GetHashCode()
+		{
+			return base.GetHashCode();
+		}
+
 	}
-}
\ No newline at end of file
+
+}
+
