@@ -48,7 +48,10 @@ namespace Boo.Lang.Compiler.TypeSystem
 		protected ExpressionCollection _returnExpressions;
 
 		protected List _yieldStatements;
-				
+
+		private bool? _isBooExtension;
+		private bool? _isClrExtension;
+
 		internal InternalMethod(TypeSystemServices typeSystemServices, Method method)
 		{
 			_typeSystemServices = typeSystemServices;
@@ -69,12 +72,32 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			get
 			{
-				bool defined = IsAttributeDefined(Types.BooExtensionAttribute);
-				if( defined == false && Types.ClrExtensionAttribute != null )
+				return IsBooExtension || IsClrExtension;
+			}
+		}
+
+		public bool IsBooExtension
+		{
+			get
+			{
+				if (null == _isBooExtension)
 				{
-					defined = IsAttributeDefined(Types.ClrExtensionAttribute);
+					_isBooExtension = IsAttributeDefined(Types.BooExtensionAttribute);
 				}
-				return defined;
+				return _isBooExtension.Value;
+			}
+		}
+
+		public bool IsClrExtension
+		{
+			get
+			{
+				if (null == _isClrExtension)
+				{
+					_isClrExtension = MetadataUtil.HasClrExtensions()
+							&& IsAttributeDefined(Types.ClrExtensionAttribute);
+				}
+				return _isClrExtension.Value;
 			}
 		}
 
@@ -94,10 +117,9 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 		
-		bool IsAttributeDefined(System.Type attributeType)
+		private bool IsAttributeDefined(System.Type attributeType)
 		{
-			IType entity = _typeSystemServices.Map(attributeType);
-			return MetadataUtil.IsAttributeDefined(_method, entity);
+			return MetadataUtil.IsAttributeDefined(_method, _typeSystemServices.Map(attributeType));
 		}
 		
 		public IType DeclaringType
