@@ -32,28 +32,62 @@ namespace Boo.Lang.Compiler.TypeSystem
 
 	public class InternalProperty : IInternalEntity, IProperty
 	{
-		TypeSystemServices _typeSystemServices;
+		private TypeSystemServices _typeSystemServices;
 		
-		Property _property;
+		private Property _property;
 		
-		IParameter[] _parameters;
+		private IParameter[] _parameters;
 		
-		IProperty _override;
-		
-		public InternalProperty(TypeSystemServices tagManager, Property property)
+		private IProperty _override;
+
+		private bool? _isBooExtension;
+		private bool? _isClrExtension;
+
+		public InternalProperty(TypeSystemServices typeSystemServices, Property property)
 		{
-			_typeSystemServices = tagManager;
+			_typeSystemServices = typeSystemServices;
 			_property = property;
 		}
-		
+
 		public bool IsExtension
 		{
 			get
 			{
-				return MetadataUtil.IsAttributeDefined(_property, _typeSystemServices.Map(Types.BooExtensionAttribute));
+				return IsBooExtension || IsClrExtension;
 			}
 		}
-		
+
+		public bool IsBooExtension
+		{
+			get
+			{
+				if (null == _isBooExtension)
+				{
+					_isBooExtension = IsAttributeDefined(Types.BooExtensionAttribute);
+				}
+				return _isBooExtension.Value;
+			}
+		}
+
+		public bool IsClrExtension
+		{
+			get
+			{
+				if (null == _isClrExtension)
+				{
+					_isClrExtension = MetadataUtil.HasClrExtensions()
+							&& IsStatic
+							&& IsAttributeDefined(Types.ClrExtensionAttribute);
+				}
+				return _isClrExtension.Value;
+			}
+		}
+
+		private bool IsAttributeDefined(System.Type attributeType)
+		{
+			return MetadataUtil.IsAttributeDefined(_property, _typeSystemServices.Map(attributeType));
+		}
+
 		public IType DeclaringType
 		{
 			get
