@@ -224,25 +224,31 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 			return false;
 		}
-		
+
 		public void ResolveTypeReference(TypeReference node)
 		{
-			if (NodeType.ArrayTypeReference == node.NodeType)
+			if (null != node.Entity) return;
+
+			switch (node.NodeType)
 			{
-				ResolveArrayTypeReference((ArrayTypeReference)node);
-			}
-			else
-			{
-				ResolveSimpleTypeReference((SimpleTypeReference)node);
+				case NodeType.ArrayTypeReference:
+					ResolveArrayTypeReference((ArrayTypeReference) node);
+					break;
+
+				case NodeType.CallableTypeReference:
+					//not needed? (late resolution)
+					//ResolveCallableTypeReference((CallableTypeReference) node);
+					break;
+
+				default:
+					ResolveSimpleTypeReference((SimpleTypeReference) node);
+					break;
 			}
 		}
-		
+
 		public void ResolveArrayTypeReference(ArrayTypeReference node)
 		{
-			if (node.Entity != null)
-			{
-				return;
-			}
+			if (null != node.Entity) return;
 
 			ResolveTypeReference(node.ElementType);
 			
@@ -257,7 +263,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 				node.Entity = _context.TypeSystemServices.GetArrayType(elementType, rank);
 			}
 		}
-		
+
 		private void ResolveTypeReferenceCollection(TypeReferenceCollection collection)
 		{
 			foreach (TypeReference tr in collection)
@@ -461,13 +467,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			string lastNs = "!!not a namespace!!";
 			NamespaceEntity lastNsEntity = null;
-			string ns;
 
 			foreach (Type type in types)
 			{
 				if (!type.IsPublic) continue;
 
-				ns = type.Namespace ?? string.Empty;
+				string ns = type.Namespace ?? string.Empty;
 				//retrieve the namespace only if we don't have it handy already
 				//usually we'll have it since GetExportedTypes() seems to export
 				//types in a sorted fashion.
