@@ -56,7 +56,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			_parent = parent;
 			_typeSystemServices = tagManager;
 			_name = name;
-			_assemblies = new Dictionary<Assembly, Dictionary<string, List<Type>>>();
+			_assemblies = new Dictionary<Assembly, Dictionary<string, List<Type>>>(AssemblyEqualityComparer.Default);
 			_childrenNamespaces = new Dictionary<string, NamespaceEntity>();
 			_internalModules = new List<ModuleEntity>();
 			_externalModules = new List<ExternalType>();
@@ -149,27 +149,26 @@ namespace Boo.Lang.Compiler.TypeSystem
 		internal bool Resolve(List targetList, string name, Assembly assembly, EntityType flags)
 		{
 			NamespaceEntity entity;
-			_childrenNamespaces.TryGetValue(name, out entity);
-			if (null != entity)
+			if (_childrenNamespaces.TryGetValue(name, out entity))
 			{
 				targetList.Add(new AssemblyQualifiedNamespaceEntity(assembly, entity));
 				return true;
 			}
+
 			
 			Dictionary<string, List<Type>> types;
-			_assemblies.TryGetValue(assembly, out types);
-			
-			bool found = false;
-			if (null != types)
+			if (!_assemblies.TryGetValue(assembly, out types))
 			{
-				found = ResolveType(targetList, name, types);
-				
-				foreach (ExternalType external in _externalModules)
+				return false;
+			}
+
+
+			bool found = ResolveType(targetList, name, types);
+			foreach (ExternalType external in _externalModules)
+			{
+				if (AssemblyEqualityComparer.Default.Equals(external.ActualType.Assembly, assembly))
 				{
-					if (external.ActualType.Assembly == assembly)
-					{
-						if (external.Resolve(targetList, name, flags)) found = true; 
-					}
+					if (external.Resolve(targetList, name, flags)) found = true;
 				}
 			}
 			return found;
@@ -216,7 +215,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			MemberInfo[] members = MetadataUtil.GetClrExtensions(type, name);
 			if (0 == members.Length) return false;
-					
+
 			foreach (MethodInfo method in members)
 			{
 				list.Add(_typeSystemServices.Map(method));
@@ -268,7 +267,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		bool ResolveExternalModules(List targetList, string name, EntityType flags)
 		{
 			bool found = false;
-			foreach (INamespace ns in _externalModules)
+			foreach (ExternalType ns in _externalModules)
 			{
 				if (ns.Resolve(targetList, name, flags)) found = true;
 			}
