@@ -36,83 +36,79 @@ namespace Boo.Lang.Compiler.TypeSystem
 	public class Types
 	{
 		public static readonly Type RuntimeServices = typeof(RuntimeServices);
-		
+
 		public static readonly Type Builtins = typeof(Builtins);
-		
-		public static readonly Type Exception = typeof(Exception);
-		
-		public static readonly Type ApplicationException = typeof(ApplicationException);
-		
+
 		public static readonly Type List = typeof(List);
-		
+
 		public static readonly Type Hash = typeof(Hash);
-		
+
 		public static readonly Type ICallable = typeof(ICallable);
-		
+
 		public static readonly Type ICollection = typeof(ICollection);
-		
+
 		public static readonly Type IList = typeof(IList);
-		
+
 		public static readonly Type IDictionary = typeof(IDictionary);
-		
+
 		public static readonly Type IEnumerable = typeof(IEnumerable);
-		
+
 		public static readonly Type IEnumerator = typeof(IEnumerator);
-		
+
 		public static readonly Type Object = typeof(object);
-		
+
 		public static readonly Type Regex = typeof(Regex);
-		
+
 		public static readonly Type ValueType = typeof(ValueType);
-		
+
 		public static readonly Type Array = typeof(Array);
-		
+
 		public static readonly Type ObjectArray = Type.GetType("System.Object[]");
-		
+
 		public static readonly Type Void = typeof(void);
-		
+
 		public static readonly Type String = typeof(string);
-		
+
 		public static readonly Type Byte = typeof(byte);
-		
+
 		public static readonly Type SByte = typeof(sbyte);
-		
+
 		public static readonly Type Char = typeof(char);
-		
+
 		public static readonly Type Short = typeof(short);
-		
+
 		public static readonly Type Int = typeof(int);
-		
+
 		public static readonly Type Long = typeof(long);
-		
+
 		public static readonly Type UShort = typeof(ushort);
-		
+
 		public static readonly Type UInt = typeof(uint);
-		
+
 		public static readonly Type ULong = typeof(ulong);
-		
+
 		public static readonly Type TimeSpan = typeof(TimeSpan);
-		
+
 		public static readonly Type DateTime = typeof(DateTime);
-		
+
 		public static readonly Type Single = typeof(float);
-		
+
 		public static readonly Type Double = typeof(double);
-		
+
 		public static readonly Type Decimal = typeof(decimal);
-		
+
 		public static readonly Type Date = typeof(DateTime);
-		
+
 		public static readonly Type Bool = typeof(bool);
-		
+
 		public static readonly Type IntPtr = typeof(IntPtr);
-		
+
 		public static readonly Type UIntPtr = typeof(UIntPtr);
-		
+
 		public static readonly Type Type = typeof(Type);
-		
+
 		public static readonly Type MulticastDelegate = typeof(MulticastDelegate);
-		
+
 		public static readonly Type Delegate = typeof(Delegate);
 
 		public static readonly Type DuckTypedAttribute = typeof(Boo.Lang.DuckTypedAttribute);
@@ -120,14 +116,17 @@ namespace Boo.Lang.Compiler.TypeSystem
 		public static readonly Type BooExtensionAttribute = typeof(Boo.Lang.ExtensionAttribute);
 
 		public static readonly Type ClrExtensionAttribute = Type.GetType("System.Runtime.CompilerServices.ExtensionAttribute, System.Core, Version=3.5.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
-		
+
 		public static readonly Type DllImportAttribute = typeof(System.Runtime.InteropServices.DllImportAttribute);
 
 		public static readonly Type ModuleAttribute = typeof(System.Runtime.CompilerServices.CompilerGlobalScopeAttribute);
 
 		public static readonly Type ParamArrayAttribute = typeof(ParamArrayAttribute);
-		
+
 		public static readonly Type DefaultMemberAttribute = typeof(System.Reflection.DefaultMemberAttribute);
 
+		public static readonly Type Nullable = typeof(Nullable<>);
+
 	}
 }
+
