@@ -39,8 +39,3 @@ using System.Reflection;
 [assembly: AssemblyVersion("2.0.0.0")]
 [assembly: AssemblyDelaySign(false)]
 
-#if !IGNOREKEYFILE
-[assembly: AssemblyKeyFile("../src/boo.snk")]
-#endif
-
-[assembly: AssemblyKeyName("")]
