@@ -93,79 +93,81 @@ namespace Boo.Lang.Parser
 		public const int SUPER = 59;
 		public const int STATIC = 60;
 		public const int STRUCT = 61;
-		public const int TRY = 62;
-		public const int TRANSIENT = 63;
-		public const int TRUE = 64;
-		public const int TYPEOF = 65;
-		public const int UNLESS = 66;
-		public const int VIRTUAL = 67;
-		public const int WHILE = 68;
-		public const int YIELD = 69;
-		public const int TRIPLE_QUOTED_STRING = 70;
-		public const int EOS = 71;
-		public const int DOUBLE_QUOTED_STRING = 72;
-		public const int SINGLE_QUOTED_STRING = 73;
-		public const int ID = 74;
-		public const int LBRACK = 75;
-		public const int RBRACK = 76;
-		public const int LPAREN = 77;
-		public const int RPAREN = 78;
-		public const int ASSIGN = 79;
-		public const int SUBTRACT = 80;
-		public const int COMMA = 81;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 82;
-		public const int SPLICE_BEGIN = 83;
-		public const int DOT = 84;
-		public const int COLON = 85;
-		public const int MULTIPLY = 86;
-		public const int BITWISE_OR = 87;
-		public const int LBRACE = 88;
-		public const int RBRACE = 89;
-		public const int QQ_BEGIN = 90;
-		public const int QQ_END = 91;
-		public const int INPLACE_BITWISE_OR = 92;
-		public const int INPLACE_BITWISE_AND = 93;
-		public const int INPLACE_SHIFT_LEFT = 94;
-		public const int INPLACE_SHIFT_RIGHT = 95;
-		public const int CMP_OPERATOR = 96;
-		public const int GREATER_THAN = 97;
-		public const int LESS_THAN = 98;
-		public const int ADD = 99;
-		public const int EXCLUSIVE_OR = 100;
-		public const int DIVISION = 101;
-		public const int MODULUS = 102;
-		public const int BITWISE_AND = 103;
-		public const int SHIFT_LEFT = 104;
-		public const int SHIFT_RIGHT = 105;
-		public const int EXPONENTIATION = 106;
-		public const int INCREMENT = 107;
-		public const int DECREMENT = 108;
-		public const int ONES_COMPLEMENT = 109;
-		public const int INT = 110;
-		public const int LONG = 111;
-		public const int RE_LITERAL = 112;
-		public const int DOUBLE = 113;
-		public const int FLOAT = 114;
-		public const int TIMESPAN = 115;
-		public const int LINE_CONTINUATION = 116;
-		public const int SL_COMMENT = 117;
-		public const int ML_COMMENT = 118;
-		public const int WS = 119;
-		public const int X_RE_LITERAL = 120;
-		public const int NEWLINE = 121;
-		public const int ESCAPED_EXPRESSION = 122;
-		public const int DQS_ESC = 123;
-		public const int SQS_ESC = 124;
-		public const int SESC = 125;
-		public const int RE_CHAR = 126;
-		public const int X_RE_CHAR = 127;
-		public const int RE_ESC = 128;
-		public const int DIGIT_GROUP = 129;
-		public const int REVERSE_DIGIT_GROUP = 130;
-		public const int ID_PREFIX = 131;
-		public const int ID_LETTER = 132;
-		public const int DIGIT = 133;
-		public const int HEXDIGIT = 134;
+		public const int THEN = 62;
+		public const int TRY = 63;
+		public const int TRANSIENT = 64;
+		public const int TRUE = 65;
+		public const int TYPEOF = 66;
+		public const int UNLESS = 67;
+		public const int VIRTUAL = 68;
+		public const int WHILE = 69;
+		public const int YIELD = 70;
+		public const int TRIPLE_QUOTED_STRING = 71;
+		public const int EOS = 72;
+		public const int DOUBLE_QUOTED_STRING = 73;
+		public const int SINGLE_QUOTED_STRING = 74;
+		public const int ID = 75;
+		public const int LBRACK = 76;
+		public const int RBRACK = 77;
+		public const int LPAREN = 78;
+		public const int RPAREN = 79;
+		public const int ASSIGN = 80;
+		public const int SUBTRACT = 81;
+		public const int COMMA = 82;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 83;
+		public const int SPLICE_BEGIN = 84;
+		public const int DOT = 85;
+		public const int COLON = 86;
+		public const int MULTIPLY = 87;
+		public const int NULLABLE_SUFFIX = 88;
+		public const int BITWISE_OR = 89;
+		public const int LBRACE = 90;
+		public const int RBRACE = 91;
+		public const int QQ_BEGIN = 92;
+		public const int QQ_END = 93;
+		public const int INPLACE_BITWISE_OR = 94;
+		public const int INPLACE_BITWISE_AND = 95;
+		public const int INPLACE_SHIFT_LEFT = 96;
+		public const int INPLACE_SHIFT_RIGHT = 97;
+		public const int CMP_OPERATOR = 98;
+		public const int GREATER_THAN = 99;
+		public const int LESS_THAN = 100;
+		public const int ADD = 101;
+		public const int EXCLUSIVE_OR = 102;
+		public const int DIVISION = 103;
+		public const int MODULUS = 104;
+		public const int BITWISE_AND = 105;
+		public const int SHIFT_LEFT = 106;
+		public const int SHIFT_RIGHT = 107;
+		public const int EXPONENTIATION = 108;
+		public const int INCREMENT = 109;
+		public const int DECREMENT = 110;
+		public const int ONES_COMPLEMENT = 111;
+		public const int INT = 112;
+		public const int LONG = 113;
+		public const int RE_LITERAL = 114;
+		public const int DOUBLE = 115;
+		public const int FLOAT = 116;
+		public const int TIMESPAN = 117;
+		public const int LINE_CONTINUATION = 118;
+		public const int SL_COMMENT = 119;
+		public const int ML_COMMENT = 120;
+		public const int WS = 121;
+		public const int X_RE_LITERAL = 122;
+		public const int NEWLINE = 123;
+		public const int ESCAPED_EXPRESSION = 124;
+		public const int DQS_ESC = 125;
+		public const int SQS_ESC = 126;
+		public const int SESC = 127;
+		public const int RE_CHAR = 128;
+		public const int X_RE_CHAR = 129;
+		public const int RE_ESC = 130;
+		public const int DIGIT_GROUP = 131;
+		public const int REVERSE_DIGIT_GROUP = 132;
+		public const int ID_PREFIX = 133;
+		public const int ID_LETTER = 134;
+		public const int DIGIT = 135;
+		public const int HEXDIGIT = 136;
 		
 	}
 }
