@@ -123,79 +123,81 @@ using Boo.Lang.Parser.Util;
 		public const int SUPER = 59;
 		public const int STATIC = 60;
 		public const int STRUCT = 61;
-		public const int TRY = 62;
-		public const int TRANSIENT = 63;
-		public const int TRUE = 64;
-		public const int TYPEOF = 65;
-		public const int UNLESS = 66;
-		public const int VIRTUAL = 67;
-		public const int WHILE = 68;
-		public const int YIELD = 69;
-		public const int TRIPLE_QUOTED_STRING = 70;
-		public const int EOS = 71;
-		public const int DOUBLE_QUOTED_STRING = 72;
-		public const int SINGLE_QUOTED_STRING = 73;
-		public const int ID = 74;
-		public const int LBRACK = 75;
-		public const int RBRACK = 76;
-		public const int LPAREN = 77;
-		public const int RPAREN = 78;
-		public const int ASSIGN = 79;
-		public const int SUBTRACT = 80;
-		public const int COMMA = 81;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 82;
-		public const int SPLICE_BEGIN = 83;
-		public const int DOT = 84;
-		public const int COLON = 85;
-		public const int MULTIPLY = 86;
-		public const int BITWISE_OR = 87;
-		public const int LBRACE = 88;
-		public const int RBRACE = 89;
-		public const int QQ_BEGIN = 90;
-		public const int QQ_END = 91;
-		public const int INPLACE_BITWISE_OR = 92;
-		public const int INPLACE_BITWISE_AND = 93;
-		public const int INPLACE_SHIFT_LEFT = 94;
-		public const int INPLACE_SHIFT_RIGHT = 95;
-		public const int CMP_OPERATOR = 96;
-		public const int GREATER_THAN = 97;
-		public const int LESS_THAN = 98;
-		public const int ADD = 99;
-		public const int EXCLUSIVE_OR = 100;
-		public const int DIVISION = 101;
-		public const int MODULUS = 102;
-		public const int BITWISE_AND = 103;
-		public const int SHIFT_LEFT = 104;
-		public const int SHIFT_RIGHT = 105;
-		public const int EXPONENTIATION = 106;
-		public const int INCREMENT = 107;
-		public const int DECREMENT = 108;
-		public const int ONES_COMPLEMENT = 109;
-		public const int INT = 110;
-		public const int LONG = 111;
-		public const int RE_LITERAL = 112;
-		public const int DOUBLE = 113;
-		public const int FLOAT = 114;
-		public const int TIMESPAN = 115;
-		public const int LINE_CONTINUATION = 116;
-		public const int SL_COMMENT = 117;
-		public const int ML_COMMENT = 118;
-		public const int WS = 119;
-		public const int X_RE_LITERAL = 120;
-		public const int NEWLINE = 121;
-		public const int ESCAPED_EXPRESSION = 122;
-		public const int DQS_ESC = 123;
-		public const int SQS_ESC = 124;
-		public const int SESC = 125;
-		public const int RE_CHAR = 126;
-		public const int X_RE_CHAR = 127;
-		public const int RE_ESC = 128;
-		public const int DIGIT_GROUP = 129;
-		public const int REVERSE_DIGIT_GROUP = 130;
-		public const int ID_PREFIX = 131;
-		public const int ID_LETTER = 132;
-		public const int DIGIT = 133;
-		public const int HEXDIGIT = 134;
+		public const int THEN = 62;
+		public const int TRY = 63;
+		public const int TRANSIENT = 64;
+		public const int TRUE = 65;
+		public const int TYPEOF = 66;
+		public const int UNLESS = 67;
+		public const int VIRTUAL = 68;
+		public const int WHILE = 69;
+		public const int YIELD = 70;
+		public const int TRIPLE_QUOTED_STRING = 71;
+		public const int EOS = 72;
+		public const int DOUBLE_QUOTED_STRING = 73;
+		public const int SINGLE_QUOTED_STRING = 74;
+		public const int ID = 75;
+		public const int LBRACK = 76;
+		public const int RBRACK = 77;
+		public const int LPAREN = 78;
+		public const int RPAREN = 79;
+		public const int ASSIGN = 80;
+		public const int SUBTRACT = 81;
+		public const int COMMA = 82;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 83;
+		public const int SPLICE_BEGIN = 84;
+		public const int DOT = 85;
+		public const int COLON = 86;
+		public const int MULTIPLY = 87;
+		public const int NULLABLE_SUFFIX = 88;
+		public const int BITWISE_OR = 89;
+		public const int LBRACE = 90;
+		public const int RBRACE = 91;
+		public const int QQ_BEGIN = 92;
+		public const int QQ_END = 93;
+		public const int INPLACE_BITWISE_OR = 94;
+		public const int INPLACE_BITWISE_AND = 95;
+		public const int INPLACE_SHIFT_LEFT = 96;
+		public const int INPLACE_SHIFT_RIGHT = 97;
+		public const int CMP_OPERATOR = 98;
+		public const int GREATER_THAN = 99;
+		public const int LESS_THAN = 100;
+		public const int ADD = 101;
+		public const int EXCLUSIVE_OR = 102;
+		public const int DIVISION = 103;
+		public const int MODULUS = 104;
+		public const int BITWISE_AND = 105;
+		public const int SHIFT_LEFT = 106;
+		public const int SHIFT_RIGHT = 107;
+		public const int EXPONENTIATION = 108;
+		public const int INCREMENT = 109;
+		public const int DECREMENT = 110;
+		public const int ONES_COMPLEMENT = 111;
+		public const int INT = 112;
+		public const int LONG = 113;
+		public const int RE_LITERAL = 114;
+		public const int DOUBLE = 115;
+		public const int FLOAT = 116;
+		public const int TIMESPAN = 117;
+		public const int LINE_CONTINUATION = 118;
+		public const int SL_COMMENT = 119;
+		public const int ML_COMMENT = 120;
+		public const int WS = 121;
+		public const int X_RE_LITERAL = 122;
+		public const int NEWLINE = 123;
+		public const int ESCAPED_EXPRESSION = 124;
+		public const int DQS_ESC = 125;
+		public const int SQS_ESC = 126;
+		public const int SESC = 127;
+		public const int RE_CHAR = 128;
+		public const int X_RE_CHAR = 129;
+		public const int RE_ESC = 130;
+		public const int DIGIT_GROUP = 131;
+		public const int REVERSE_DIGIT_GROUP = 132;
+		public const int ID_PREFIX = 133;
+		public const int ID_LETTER = 134;
+		public const int DIGIT = 135;
+		public const int HEXDIGIT = 136;
 		
 		
 	protected int _skipWhitespaceRegion = 0;
@@ -280,9 +282,10 @@ using Boo.Lang.Parser.Util;
 			literals.Add("public", 51);
 			literals.Add("namespace", 49);
 			literals.Add("break", 13);
-			literals.Add("while", 68);
+			literals.Add("while", 69);
+			literals.Add("then", 62);
 			literals.Add("raise", 54);
-			literals.Add("typeof", 65);
+			literals.Add("typeof", 66);
 			literals.Add("and", 11);
 			literals.Add("failure", 29);
 			literals.Add("not", 43);
@@ -297,23 +300,23 @@ using Boo.Lang.Parser.Util;
 			literals.Add("do", 22);
 			literals.Add("except", 28);
 			literals.Add("event", 27);
-			literals.Add("unless", 66);
+			literals.Add("unless", 67);
 			literals.Add("super", 59);
 			literals.Add("set", 57);
-			literals.Add("transient", 63);
+			literals.Add("transient", 64);
 			literals.Add("constructor", 19);
 			literals.Add("interface", 37);
 			literals.Add("of", 45);
 			literals.Add("is", 39);
 			literals.Add("internal", 38);
 			literals.Add("final", 30);
-			literals.Add("yield", 69);
+			literals.Add("yield", 70);
 			literals.Add("or", 46);
 			literals.Add("destructor", 21);
 			literals.Add("if", 41);
 			literals.Add("override", 47);
 			literals.Add("as", 12);
-			literals.Add("try", 62);
+			literals.Add("try", 63);
 			literals.Add("goto", 35);
 			literals.Add("enum", 26);
 			literals.Add("isa", 40);
@@ -335,8 +338,8 @@ using Boo.Lang.Parser.Util;
 			literals.Add("in", 42);
 			literals.Add("self", 58);
 			literals.Add("ensure", 25);
-			literals.Add("true", 64);
-			literals.Add("virtual", 67);
+			literals.Add("true", 65);
+			literals.Add("virtual", 68);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -489,6 +492,12 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
+						case '?':
+						{
+							mNULLABLE_SUFFIX(true);
+							theRetToken = returnToken_;
+							break;
+						}
 						default:
 							if ((cached_LA1=='<') && (cached_LA2=='<') && (LA(3)=='='))
 							{
@@ -599,7 +608,7 @@ tryAgain:
 		_ttype = ID;
 		
 		{
-			if ((cached_LA1=='?'||cached_LA1=='@'))
+			if ((cached_LA1=='@'))
 			{
 				mID_PREFIX(false);
 			}
@@ -624,11 +633,11 @@ tryAgain:
 				}
 				else
 				{
-					goto _loop607_breakloop;
+					goto _loop617_breakloop;
 				}
 				
 			}
-_loop607_breakloop:			;
+_loop617_breakloop:			;
 		}    // ( ... )*
 		_ttype = testLiteralsTable(_ttype);
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -644,23 +653,7 @@ _loop607_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = ID_PREFIX;
 		
-		switch ( cached_LA1 )
-		{
-		case '@':
-		{
-			match('@');
-			break;
-		}
-		case '?':
-		{
-			match('?');
-			break;
-		}
-		default:
-		{
-			throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
-		}
-		 }
+		match('@');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
 			_token = makeToken(_ttype);
@@ -816,7 +809,7 @@ _loop607_breakloop:			;
 			{
 				match("0x");
 				{ // ( ... )+
-					int _cnt612=0;
+					int _cnt622=0;
 					for (;;)
 					{
 						if ((tokenSet_2_.member(cached_LA1)))
@@ -825,12 +818,12 @@ _loop607_breakloop:			;
 						}
 						else
 						{
-							if (_cnt612 >= 1) { goto _loop612_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt622 >= 1) { goto _loop622_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt612++;
+						_cnt622++;
 					}
-_loop612_breakloop:					;
+_loop622_breakloop:					;
 				}    // ( ... )+
 			}
 			{
@@ -1220,11 +1213,11 @@ _loop612_breakloop:					;
 				}
 				default:
 				{
-					goto _loop770_breakloop;
+					goto _loop780_breakloop;
 				}
 				 }
 			}
-_loop770_breakloop:			;
+_loop780_breakloop:			;
 		}    // ( ... )*
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1240,7 +1233,7 @@ _loop770_breakloop:			;
 		_ttype = REVERSE_DIGIT_GROUP;
 		
 		{ // ( ... )+
-			int _cnt774=0;
+			int _cnt784=0;
 			for (;;)
 			{
 				if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && ((cached_LA2 >= '0' && cached_LA2 <= '9')) && ((LA(3) >= '0' && LA(3) <= '9')))
@@ -1266,12 +1259,12 @@ _loop770_breakloop:			;
 				}
 				else
 				{
-					if (_cnt774 >= 1) { goto _loop774_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt784 >= 1) { goto _loop784_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt774++;
+				_cnt784++;
 			}
-_loop774_breakloop:			;
+_loop784_breakloop:			;
 		}    // ( ... )+
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1596,11 +1589,11 @@ _loop774_breakloop:			;
 			EnterSkipWhitespaceRegion();
 		}
 		{
-			bool synPredMatched656 = false;
+			bool synPredMatched666 = false;
 			if (((cached_LA1=='a')))
 			{
-				int _m656 = mark();
-				synPredMatched656 = true;
+				int _m666 = mark();
+				synPredMatched666 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1609,12 +1602,12 @@ _loop774_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched656 = false;
+					synPredMatched666 = false;
 				}
-				rewind(_m656);
+				rewind(_m666);
 				inputState.guessing--;
 			}
-			if ( synPredMatched656 )
+			if ( synPredMatched666 )
 			{
 				match("assembly:");
 				if (0==inputState.guessing)
@@ -1876,11 +1869,11 @@ _loop774_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = DIVISION;
 		
-		bool synPredMatched677 = false;
+		bool synPredMatched687 = false;
 		if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 		{
-			int _m677 = mark();
-			synPredMatched677 = true;
+			int _m687 = mark();
+			synPredMatched687 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1889,12 +1882,12 @@ _loop774_breakloop:			;
 			}
 			catch (RecognitionException)
 			{
-				synPredMatched677 = false;
+				synPredMatched687 = false;
 			}
-			rewind(_m677);
+			rewind(_m687);
 			inputState.guessing--;
 		}
-		if ( synPredMatched677 )
+		if ( synPredMatched687 )
 		{
 			mML_COMMENT(false);
 			if (0==inputState.guessing)
@@ -1903,11 +1896,11 @@ _loop774_breakloop:			;
 			}
 		}
 		else {
-			bool synPredMatched679 = false;
+			bool synPredMatched689 = false;
 			if (((cached_LA1=='/') && (tokenSet_3_.member(cached_LA2)) && (tokenSet_4_.member(LA(3)))))
 			{
-				int _m679 = mark();
-				synPredMatched679 = true;
+				int _m689 = mark();
+				synPredMatched689 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1916,12 +1909,12 @@ _loop774_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched679 = false;
+					synPredMatched689 = false;
 				}
-				rewind(_m679);
+				rewind(_m689);
 				inputState.guessing--;
 			}
-			if ( synPredMatched679 )
+			if ( synPredMatched689 )
 			{
 				mRE_LITERAL(false);
 				if (0==inputState.guessing)
@@ -1949,11 +1942,11 @@ _loop774_breakloop:			;
 									}
 									else
 									{
-										goto _loop684_breakloop;
+										goto _loop694_breakloop;
 									}
 									
 								}
-_loop684_breakloop:								;
+_loop694_breakloop:								;
 							}    // ( ... )*
 							if (0==inputState.guessing)
 							{
@@ -2006,11 +1999,11 @@ _loop684_breakloop:								;
 					match('*');
 				}
 				else {
-					bool synPredMatched725 = false;
+					bool synPredMatched735 = false;
 					if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m725 = mark();
-						synPredMatched725 = true;
+						int _m735 = mark();
+						synPredMatched735 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2019,12 +2012,12 @@ _loop684_breakloop:								;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched725 = false;
+							synPredMatched735 = false;
 						}
-						rewind(_m725);
+						rewind(_m735);
 						inputState.guessing--;
 					}
-					if ( synPredMatched725 )
+					if ( synPredMatched735 )
 					{
 						mML_COMMENT(false);
 					}
@@ -2038,11 +2031,11 @@ _loop684_breakloop:								;
 					}
 					else
 					{
-						goto _loop727_breakloop;
+						goto _loop737_breakloop;
 					}
 					}
 				}
-_loop727_breakloop:				;
+_loop737_breakloop:				;
 			}    // ( ... )*
 			match("*/");
 			if (0==inputState.guessing)
@@ -2064,7 +2057,7 @@ _loop727_breakloop:				;
 		
 		match('/');
 		{ // ( ... )+
-			int _cnt756=0;
+			int _cnt766=0;
 			for (;;)
 			{
 				if ((tokenSet_3_.member(cached_LA1)))
@@ -2073,12 +2066,12 @@ _loop727_breakloop:				;
 				}
 				else
 				{
-					if (_cnt756 >= 1) { goto _loop756_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt766 >= 1) { goto _loop766_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt756++;
+				_cnt766++;
 			}
-_loop756_breakloop:			;
+_loop766_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -2297,12 +2290,12 @@ _loop756_breakloop:			;
 			for (;;)
 			{
 				// nongreedy exit test
-				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop705_breakloop;
-				bool synPredMatched701 = false;
+				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop715_breakloop;
+				bool synPredMatched711 = false;
 				if (((cached_LA1=='$') && (cached_LA2=='{') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 				{
-					int _m701 = mark();
-					synPredMatched701 = true;
+					int _m711 = mark();
+					synPredMatched711 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -2311,12 +2304,12 @@ _loop756_breakloop:			;
 					}
 					catch (RecognitionException)
 					{
-						synPredMatched701 = false;
+						synPredMatched711 = false;
 					}
-					rewind(_m701);
+					rewind(_m711);
 					inputState.guessing--;
 				}
-				if ( synPredMatched701 )
+				if ( synPredMatched711 )
 				{
 					if (0==inputState.guessing)
 					{
@@ -2328,11 +2321,11 @@ _loop756_breakloop:			;
 					mESCAPED_EXPRESSION(false);
 				}
 				else {
-					bool synPredMatched703 = false;
+					bool synPredMatched713 = false;
 					if (((cached_LA1=='\\') && (cached_LA2=='$') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m703 = mark();
-						synPredMatched703 = true;
+						int _m713 = mark();
+						synPredMatched713 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2341,12 +2334,12 @@ _loop756_breakloop:			;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched703 = false;
+							synPredMatched713 = false;
 						}
-						rewind(_m703);
+						rewind(_m713);
 						inputState.guessing--;
 					}
-					if ( synPredMatched703 )
+					if ( synPredMatched713 )
 					{
 						_saveIndex = text.Length;
 						match('\\');
@@ -2363,11 +2356,11 @@ _loop756_breakloop:			;
 					}
 					else
 					{
-						goto _loop705_breakloop;
+						goto _loop715_breakloop;
 					}
 					}
 				}
-_loop705_breakloop:				;
+_loop715_breakloop:				;
 			}    // ( ... )*
 			_saveIndex = text.Length;
 			match("\"\"\"");
@@ -2435,11 +2428,11 @@ _loop705_breakloop:				;
 					{    // ( ... )*
 						for (;;)
 						{
-							bool synPredMatched711 = false;
+							bool synPredMatched721 = false;
 							if (((cached_LA1=='$') && (cached_LA2=='{') && (tokenSet_5_.member(LA(3)))))
 							{
-								int _m711 = mark();
-								synPredMatched711 = true;
+								int _m721 = mark();
+								synPredMatched721 = true;
 								inputState.guessing++;
 								try {
 									{
@@ -2448,12 +2441,12 @@ _loop705_breakloop:				;
 								}
 								catch (RecognitionException)
 								{
-									synPredMatched711 = false;
+									synPredMatched721 = false;
 								}
-								rewind(_m711);
+								rewind(_m721);
 								inputState.guessing--;
 							}
-							if ( synPredMatched711 )
+							if ( synPredMatched721 )
 							{
 								if (0==inputState.guessing)
 								{
@@ -2474,11 +2467,11 @@ _loop705_breakloop:				;
 							}
 							else
 							{
-								goto _loop713_breakloop;
+								goto _loop723_breakloop;
 							}
 							
 						}
-_loop713_breakloop:						;
+_loop723_breakloop:						;
 					}    // ( ... )*
 					_saveIndex = text.Length;
 					match('"');
@@ -2577,11 +2570,11 @@ _loop713_breakloop:						;
 				}
 				else
 				{
-					goto _loop717_breakloop;
+					goto _loop727_breakloop;
 				}
 				
 			}
-_loop717_breakloop:			;
+_loop727_breakloop:			;
 		}    // ( ... )*
 		_saveIndex = text.Length;
 		match('\'');
@@ -2649,11 +2642,11 @@ _loop717_breakloop:			;
 				}
 				else
 				{
-					goto _loop721_breakloop;
+					goto _loop731_breakloop;
 				}
 				
 			}
-_loop721_breakloop:			;
+_loop731_breakloop:			;
 		}    // ( ... )*
 		if (0==inputState.guessing)
 		{
@@ -2673,7 +2666,7 @@ _loop721_breakloop:			;
 		_ttype = WS;
 		
 		{ // ( ... )+
-			int _cnt730=0;
+			int _cnt740=0;
 			for (;;)
 			{
 				switch ( cached_LA1 )
@@ -2700,12 +2693,12 @@ _loop721_breakloop:			;
 				}
 				default:
 				{
-					if (_cnt730 >= 1) { goto _loop730_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt740 >= 1) { goto _loop740_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				break; }
-				_cnt730++;
+				_cnt740++;
 			}
-_loop730_breakloop:			;
+_loop740_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -2749,7 +2742,7 @@ _loop730_breakloop:			;
 		text.Length = _saveIndex;
 		match('/');
 		{ // ( ... )+
-			int _cnt734=0;
+			int _cnt744=0;
 			for (;;)
 			{
 				if ((tokenSet_9_.member(cached_LA1)))
@@ -2758,12 +2751,12 @@ _loop730_breakloop:			;
 				}
 				else
 				{
-					if (_cnt734 >= 1) { goto _loop734_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt744 >= 1) { goto _loop744_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt734++;
+				_cnt744++;
 			}
-_loop734_breakloop:			;
+_loop744_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (0==inputState.guessing)
@@ -3126,7 +3119,7 @@ _loop734_breakloop:			;
 			case '8':  case '9':
 			{
 				{ // ( ... )+
-					int _cnt764=0;
+					int _cnt774=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && (tokenSet_5_.member(cached_LA2)) && (true))
@@ -3135,12 +3128,12 @@ _loop734_breakloop:			;
 						}
 						else
 						{
-							if (_cnt764 >= 1) { goto _loop764_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt774 >= 1) { goto _loop774_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt764++;
+						_cnt774++;
 					}
-_loop764_breakloop:					;
+_loop774_breakloop:					;
 				}    // ( ... )+
 				break;
 			}
@@ -3253,11 +3246,25 @@ _loop764_breakloop:					;
 		returnToken_ = _token;
 	}
 	
+	public void mNULLABLE_SUFFIX(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = NULLABLE_SUFFIX;
+		
+		match('?');
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	
 	private static long[] mk_tokenSet_0_()
 	{
 		long[] data = new long[3072];
-		data[0]=-9223372036854775808L;
+		data[0]=0L;
 		data[1]=576460745995190271L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
