@@ -117,80 +117,82 @@ namespace Boo.Lang.Parser
 		public const int SUPER = 55;
 		public const int STATIC = 56;
 		public const int STRUCT = 57;
-		public const int TRY = 58;
-		public const int TRANSIENT = 59;
-		public const int TRUE = 60;
-		public const int TYPEOF = 61;
-		public const int UNLESS = 62;
-		public const int VIRTUAL = 63;
-		public const int PARTIAL = 64;
-		public const int WHILE = 65;
-		public const int YIELD = 66;
-		public const int TRIPLE_QUOTED_STRING = 67;
-		public const int EOS = 68;
-		public const int NEWLINE = 69;
-		public const int DOUBLE_QUOTED_STRING = 70;
-		public const int SINGLE_QUOTED_STRING = 71;
-		public const int ID = 72;
-		public const int LBRACK = 73;
-		public const int RBRACK = 74;
-		public const int LPAREN = 75;
-		public const int RPAREN = 76;
-		public const int ASSIGN = 77;
-		public const int SUBTRACT = 78;
-		public const int COMMA = 79;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 80;
-		public const int DOT = 81;
-		public const int COLON = 82;
-		public const int MULTIPLY = 83;
-		public const int SPLICE_BEGIN = 84;
-		public const int BITWISE_OR = 85;
-		public const int LBRACE = 86;
-		public const int RBRACE = 87;
-		public const int QQ_BEGIN = 88;
-		public const int QQ_END = 89;
-		public const int INPLACE_BITWISE_OR = 90;
-		public const int INPLACE_BITWISE_AND = 91;
-		public const int INPLACE_SHIFT_LEFT = 92;
-		public const int INPLACE_SHIFT_RIGHT = 93;
-		public const int CMP_OPERATOR = 94;
-		public const int GREATER_THAN = 95;
-		public const int LESS_THAN = 96;
-		public const int ADD = 97;
-		public const int EXCLUSIVE_OR = 98;
-		public const int DIVISION = 99;
-		public const int MODULUS = 100;
-		public const int BITWISE_AND = 101;
-		public const int SHIFT_LEFT = 102;
-		public const int SHIFT_RIGHT = 103;
-		public const int EXPONENTIATION = 104;
-		public const int INCREMENT = 105;
-		public const int DECREMENT = 106;
-		public const int ONES_COMPLEMENT = 107;
-		public const int INT = 108;
-		public const int LONG = 109;
-		public const int RE_LITERAL = 110;
-		public const int DOUBLE = 111;
-		public const int FLOAT = 112;
-		public const int TIMESPAN = 113;
-		public const int LINE_CONTINUATION = 114;
-		public const int SL_COMMENT = 115;
-		public const int ML_COMMENT = 116;
-		public const int WS = 117;
-		public const int X_RE_LITERAL = 118;
-		public const int ESCAPED_EXPRESSION = 119;
-		public const int DQS_ESC = 120;
-		public const int SQS_ESC = 121;
-		public const int SESC = 122;
-		public const int RE_CHAR = 123;
-		public const int X_RE_CHAR = 124;
-		public const int RE_ESC = 125;
-		public const int DIGIT_GROUP = 126;
-		public const int REVERSE_DIGIT_GROUP = 127;
-		public const int ID_PREFIX = 128;
-		public const int ID_LETTER = 129;
-		public const int DIGIT = 130;
-		public const int HEXDIGIT = 131;
+		public const int THEN = 58;
+		public const int TRY = 59;
+		public const int TRANSIENT = 60;
+		public const int TRUE = 61;
+		public const int TYPEOF = 62;
+		public const int UNLESS = 63;
+		public const int VIRTUAL = 64;
+		public const int PARTIAL = 65;
+		public const int WHILE = 66;
+		public const int YIELD = 67;
+		public const int TRIPLE_QUOTED_STRING = 68;
+		public const int EOS = 69;
+		public const int NEWLINE = 70;
+		public const int DOUBLE_QUOTED_STRING = 71;
+		public const int SINGLE_QUOTED_STRING = 72;
+		public const int ID = 73;
+		public const int LBRACK = 74;
+		public const int RBRACK = 75;
+		public const int LPAREN = 76;
+		public const int RPAREN = 77;
+		public const int ASSIGN = 78;
+		public const int SUBTRACT = 79;
+		public const int COMMA = 80;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 81;
+		public const int DOT = 82;
+		public const int COLON = 83;
+		public const int MULTIPLY = 84;
+		public const int SPLICE_BEGIN = 85;
+		public const int NULLABLE_SUFFIX = 86;
+		public const int BITWISE_OR = 87;
+		public const int LBRACE = 88;
+		public const int RBRACE = 89;
+		public const int QQ_BEGIN = 90;
+		public const int QQ_END = 91;
+		public const int INPLACE_BITWISE_OR = 92;
+		public const int INPLACE_BITWISE_AND = 93;
+		public const int INPLACE_SHIFT_LEFT = 94;
+		public const int INPLACE_SHIFT_RIGHT = 95;
+		public const int CMP_OPERATOR = 96;
+		public const int GREATER_THAN = 97;
+		public const int LESS_THAN = 98;
+		public const int ADD = 99;
+		public const int EXCLUSIVE_OR = 100;
+		public const int DIVISION = 101;
+		public const int MODULUS = 102;
+		public const int BITWISE_AND = 103;
+		public const int SHIFT_LEFT = 104;
+		public const int SHIFT_RIGHT = 105;
+		public const int EXPONENTIATION = 106;
+		public const int INCREMENT = 107;
+		public const int DECREMENT = 108;
+		public const int ONES_COMPLEMENT = 109;
+		public const int INT = 110;
+		public const int LONG = 111;
+		public const int RE_LITERAL = 112;
+		public const int DOUBLE = 113;
+		public const int FLOAT = 114;
+		public const int TIMESPAN = 115;
+		public const int LINE_CONTINUATION = 116;
+		public const int SL_COMMENT = 117;
+		public const int ML_COMMENT = 118;
+		public const int WS = 119;
+		public const int X_RE_LITERAL = 120;
+		public const int ESCAPED_EXPRESSION = 121;
+		public const int DQS_ESC = 122;
+		public const int SQS_ESC = 123;
+		public const int SESC = 124;
+		public const int RE_CHAR = 125;
+		public const int X_RE_CHAR = 126;
+		public const int RE_ESC = 127;
+		public const int DIGIT_GROUP = 128;
+		public const int REVERSE_DIGIT_GROUP = 129;
+		public const int ID_PREFIX = 130;
+		public const int ID_LETTER = 131;
+		public const int DIGIT = 132;
+		public const int HEXDIGIT = 133;
 		
 		
 	
@@ -227,10 +229,11 @@ namespace Boo.Lang.Parser
 			literals.Add("public", 47);
 			literals.Add("namespace", 46);
 			literals.Add("break", 10);
-			literals.Add("while", 65);
+			literals.Add("while", 66);
 			literals.Add("end", 22);
+			literals.Add("then", 58);
 			literals.Add("raise", 50);
-			literals.Add("typeof", 61);
+			literals.Add("typeof", 62);
 			literals.Add("and", 8);
 			literals.Add("failure", 27);
 			literals.Add("not", 41);
@@ -244,23 +247,23 @@ namespace Boo.Lang.Parser
 			literals.Add("do", 19);
 			literals.Add("except", 26);
 			literals.Add("event", 25);
-			literals.Add("unless", 62);
+			literals.Add("unless", 63);
 			literals.Add("super", 55);
 			literals.Add("set", 53);
-			literals.Add("transient", 59);
+			literals.Add("transient", 60);
 			literals.Add("constructor", 16);
 			literals.Add("interface", 35);
 			literals.Add("of", 43);
 			literals.Add("is", 37);
 			literals.Add("internal", 36);
 			literals.Add("final", 28);
-			literals.Add("yield", 66);
+			literals.Add("yield", 67);
 			literals.Add("or", 44);
 			literals.Add("destructor", 18);
 			literals.Add("if", 39);
 			literals.Add("override", 45);
 			literals.Add("as", 9);
-			literals.Add("try", 58);
+			literals.Add("try", 59);
 			literals.Add("goto", 33);
 			literals.Add("enum", 24);
 			literals.Add("isa", 38);
@@ -270,7 +273,7 @@ namespace Boo.Lang.Parser
 			literals.Add("false", 31);
 			literals.Add("static", 56);
 			literals.Add("abstract", 7);
-			literals.Add("partial", 64);
+			literals.Add("partial", 65);
 			literals.Add("callable", 12);
 			literals.Add("get", 32);
 			literals.Add("continue", 11);
@@ -282,8 +285,8 @@ namespace Boo.Lang.Parser
 			literals.Add("in", 40);
 			literals.Add("self", 54);
 			literals.Add("ensure", 23);
-			literals.Add("true", 60);
-			literals.Add("virtual", 63);
+			literals.Add("true", 61);
+			literals.Add("virtual", 64);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -453,7 +456,7 @@ tryAgain:
 								mCMP_OPERATOR(true);
 								theRetToken = returnToken_;
 							}
-							else if ((tokenSet_0_.member(cached_LA1))) {
+							else if ((tokenSet_0_.member(cached_LA1)) && (true) && (true)) {
 								mID(true);
 								theRetToken = returnToken_;
 							}
@@ -473,6 +476,10 @@ tryAgain:
 								mGREATER_THAN(true);
 								theRetToken = returnToken_;
 							}
+							else if ((cached_LA1=='?') && (true) && (true)) {
+								mNULLABLE_SUFFIX(true);
+								theRetToken = returnToken_;
+							}
 						else
 						{
 							if (cached_LA1==EOF_CHAR) { uponEOF(); returnToken_ = makeToken(Token.EOF_TYPE); }
@@ -2504,6 +2511,20 @@ _loop111_breakloop:					;
 		returnToken_ = _token;
 	}
 	
+	public void mNULLABLE_SUFFIX(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = NULLABLE_SUFFIX;
+		
+		match('?');
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	
 	private static long[] mk_tokenSet_0_()
 	{
