@@ -89,80 +89,82 @@ namespace Boo.Lang.Parser
 		public const int SUPER = 55;
 		public const int STATIC = 56;
 		public const int STRUCT = 57;
-		public const int TRY = 58;
-		public const int TRANSIENT = 59;
-		public const int TRUE = 60;
-		public const int TYPEOF = 61;
-		public const int UNLESS = 62;
-		public const int VIRTUAL = 63;
-		public const int PARTIAL = 64;
-		public const int WHILE = 65;
-		public const int YIELD = 66;
-		public const int TRIPLE_QUOTED_STRING = 67;
-		public const int EOS = 68;
-		public const int NEWLINE = 69;
-		public const int DOUBLE_QUOTED_STRING = 70;
-		public const int SINGLE_QUOTED_STRING = 71;
-		public const int ID = 72;
-		public const int LBRACK = 73;
-		public const int RBRACK = 74;
-		public const int LPAREN = 75;
-		public const int RPAREN = 76;
-		public const int ASSIGN = 77;
-		public const int SUBTRACT = 78;
-		public const int COMMA = 79;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 80;
-		public const int DOT = 81;
-		public const int COLON = 82;
-		public const int MULTIPLY = 83;
-		public const int SPLICE_BEGIN = 84;
-		public const int BITWISE_OR = 85;
-		public const int LBRACE = 86;
-		public const int RBRACE = 87;
-		public const int QQ_BEGIN = 88;
-		public const int QQ_END = 89;
-		public const int INPLACE_BITWISE_OR = 90;
-		public const int INPLACE_BITWISE_AND = 91;
-		public const int INPLACE_SHIFT_LEFT = 92;
-		public const int INPLACE_SHIFT_RIGHT = 93;
-		public const int CMP_OPERATOR = 94;
-		public const int GREATER_THAN = 95;
-		public const int LESS_THAN = 96;
-		public const int ADD = 97;
-		public const int EXCLUSIVE_OR = 98;
-		public const int DIVISION = 99;
-		public const int MODULUS = 100;
-		public const int BITWISE_AND = 101;
-		public const int SHIFT_LEFT = 102;
-		public const int SHIFT_RIGHT = 103;
-		public const int EXPONENTIATION = 104;
-		public const int INCREMENT = 105;
-		public const int DECREMENT = 106;
-		public const int ONES_COMPLEMENT = 107;
-		public const int INT = 108;
-		public const int LONG = 109;
-		public const int RE_LITERAL = 110;
-		public const int DOUBLE = 111;
-		public const int FLOAT = 112;
-		public const int TIMESPAN = 113;
-		public const int LINE_CONTINUATION = 114;
-		public const int SL_COMMENT = 115;
-		public const int ML_COMMENT = 116;
-		public const int WS = 117;
-		public const int X_RE_LITERAL = 118;
-		public const int ESCAPED_EXPRESSION = 119;
-		public const int DQS_ESC = 120;
-		public const int SQS_ESC = 121;
-		public const int SESC = 122;
-		public const int RE_CHAR = 123;
-		public const int X_RE_CHAR = 124;
-		public const int RE_ESC = 125;
-		public const int DIGIT_GROUP = 126;
-		public const int REVERSE_DIGIT_GROUP = 127;
-		public const int ID_PREFIX = 128;
-		public const int ID_LETTER = 129;
-		public const int DIGIT = 130;
-		public const int HEXDIGIT = 131;
+		public const int THEN = 58;
+		public const int TRY = 59;
+		public const int TRANSIENT = 60;
+		public const int TRUE = 61;
+		public const int TYPEOF = 62;
+		public const int UNLESS = 63;
+		public const int VIRTUAL = 64;
+		public const int PARTIAL = 65;
+		public const int WHILE = 66;
+		public const int YIELD = 67;
+		public const int TRIPLE_QUOTED_STRING = 68;
+		public const int EOS = 69;
+		public const int NEWLINE = 70;
+		public const int DOUBLE_QUOTED_STRING = 71;
+		public const int SINGLE_QUOTED_STRING = 72;
+		public const int ID = 73;
+		public const int LBRACK = 74;
+		public const int RBRACK = 75;
+		public const int LPAREN = 76;
+		public const int RPAREN = 77;
+		public const int ASSIGN = 78;
+		public const int SUBTRACT = 79;
+		public const int COMMA = 80;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 81;
+		public const int DOT = 82;
+		public const int COLON = 83;
+		public const int MULTIPLY = 84;
+		public const int SPLICE_BEGIN = 85;
+		public const int NULLABLE_SUFFIX = 86;
+		public const int BITWISE_OR = 87;
+		public const int LBRACE = 88;
+		public const int RBRACE = 89;
+		public const int QQ_BEGIN = 90;
+		public const int QQ_END = 91;
+		public const int INPLACE_BITWISE_OR = 92;
+		public const int INPLACE_BITWISE_AND = 93;
+		public const int INPLACE_SHIFT_LEFT = 94;
+		public const int INPLACE_SHIFT_RIGHT = 95;
+		public const int CMP_OPERATOR = 96;
+		public const int GREATER_THAN = 97;
+		public const int LESS_THAN = 98;
+		public const int ADD = 99;
+		public const int EXCLUSIVE_OR = 100;
+		public const int DIVISION = 101;
+		public const int MODULUS = 102;
+		public const int BITWISE_AND = 103;
+		public const int SHIFT_LEFT = 104;
+		public const int SHIFT_RIGHT = 105;
+		public const int EXPONENTIATION = 106;
+		public const int INCREMENT = 107;
+		public const int DECREMENT = 108;
+		public const int ONES_COMPLEMENT = 109;
+		public const int INT = 110;
+		public const int LONG = 111;
+		public const int RE_LITERAL = 112;
+		public const int DOUBLE = 113;
+		public const int FLOAT = 114;
+		public const int TIMESPAN = 115;
+		public const int LINE_CONTINUATION = 116;
+		public const int SL_COMMENT = 117;
+		public const int ML_COMMENT = 118;
+		public const int WS = 119;
+		public const int X_RE_LITERAL = 120;
+		public const int ESCAPED_EXPRESSION = 121;
+		public const int DQS_ESC = 122;
+		public const int SQS_ESC = 123;
+		public const int SESC = 124;
+		public const int RE_CHAR = 125;
+		public const int X_RE_CHAR = 126;
+		public const int RE_ESC = 127;
+		public const int DIGIT_GROUP = 128;
+		public const int REVERSE_DIGIT_GROUP = 129;
+		public const int ID_PREFIX = 130;
+		public const int ID_LETTER = 131;
+		public const int DIGIT = 132;
+		public const int HEXDIGIT = 133;
 		
 	}
 }
