@@ -119,80 +119,82 @@ using Boo.Lang.Parser.Util;
 		public const int SUPER = 55;
 		public const int STATIC = 56;
 		public const int STRUCT = 57;
-		public const int TRY = 58;
-		public const int TRANSIENT = 59;
-		public const int TRUE = 60;
-		public const int TYPEOF = 61;
-		public const int UNLESS = 62;
-		public const int VIRTUAL = 63;
-		public const int PARTIAL = 64;
-		public const int WHILE = 65;
-		public const int YIELD = 66;
-		public const int TRIPLE_QUOTED_STRING = 67;
-		public const int EOS = 68;
-		public const int NEWLINE = 69;
-		public const int DOUBLE_QUOTED_STRING = 70;
-		public const int SINGLE_QUOTED_STRING = 71;
-		public const int ID = 72;
-		public const int LBRACK = 73;
-		public const int RBRACK = 74;
-		public const int LPAREN = 75;
-		public const int RPAREN = 76;
-		public const int ASSIGN = 77;
-		public const int SUBTRACT = 78;
-		public const int COMMA = 79;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 80;
-		public const int DOT = 81;
-		public const int COLON = 82;
-		public const int MULTIPLY = 83;
-		public const int SPLICE_BEGIN = 84;
-		public const int BITWISE_OR = 85;
-		public const int LBRACE = 86;
-		public const int RBRACE = 87;
-		public const int QQ_BEGIN = 88;
-		public const int QQ_END = 89;
-		public const int INPLACE_BITWISE_OR = 90;
-		public const int INPLACE_BITWISE_AND = 91;
-		public const int INPLACE_SHIFT_LEFT = 92;
-		public const int INPLACE_SHIFT_RIGHT = 93;
-		public const int CMP_OPERATOR = 94;
-		public const int GREATER_THAN = 95;
-		public const int LESS_THAN = 96;
-		public const int ADD = 97;
-		public const int EXCLUSIVE_OR = 98;
-		public const int DIVISION = 99;
-		public const int MODULUS = 100;
-		public const int BITWISE_AND = 101;
-		public const int SHIFT_LEFT = 102;
-		public const int SHIFT_RIGHT = 103;
-		public const int EXPONENTIATION = 104;
-		public const int INCREMENT = 105;
-		public const int DECREMENT = 106;
-		public const int ONES_COMPLEMENT = 107;
-		public const int INT = 108;
-		public const int LONG = 109;
-		public const int RE_LITERAL = 110;
-		public const int DOUBLE = 111;
-		public const int FLOAT = 112;
-		public const int TIMESPAN = 113;
-		public const int LINE_CONTINUATION = 114;
-		public const int SL_COMMENT = 115;
-		public const int ML_COMMENT = 116;
-		public const int WS = 117;
-		public const int X_RE_LITERAL = 118;
-		public const int ESCAPED_EXPRESSION = 119;
-		public const int DQS_ESC = 120;
-		public const int SQS_ESC = 121;
-		public const int SESC = 122;
-		public const int RE_CHAR = 123;
-		public const int X_RE_CHAR = 124;
-		public const int RE_ESC = 125;
-		public const int DIGIT_GROUP = 126;
-		public const int REVERSE_DIGIT_GROUP = 127;
-		public const int ID_PREFIX = 128;
-		public const int ID_LETTER = 129;
-		public const int DIGIT = 130;
-		public const int HEXDIGIT = 131;
+		public const int THEN = 58;
+		public const int TRY = 59;
+		public const int TRANSIENT = 60;
+		public const int TRUE = 61;
+		public const int TYPEOF = 62;
+		public const int UNLESS = 63;
+		public const int VIRTUAL = 64;
+		public const int PARTIAL = 65;
+		public const int WHILE = 66;
+		public const int YIELD = 67;
+		public const int TRIPLE_QUOTED_STRING = 68;
+		public const int EOS = 69;
+		public const int NEWLINE = 70;
+		public const int DOUBLE_QUOTED_STRING = 71;
+		public const int SINGLE_QUOTED_STRING = 72;
+		public const int ID = 73;
+		public const int LBRACK = 74;
+		public const int RBRACK = 75;
+		public const int LPAREN = 76;
+		public const int RPAREN = 77;
+		public const int ASSIGN = 78;
+		public const int SUBTRACT = 79;
+		public const int COMMA = 80;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 81;
+		public const int DOT = 82;
+		public const int COLON = 83;
+		public const int MULTIPLY = 84;
+		public const int SPLICE_BEGIN = 85;
+		public const int NULLABLE_SUFFIX = 86;
+		public const int BITWISE_OR = 87;
+		public const int LBRACE = 88;
+		public const int RBRACE = 89;
+		public const int QQ_BEGIN = 90;
+		public const int QQ_END = 91;
+		public const int INPLACE_BITWISE_OR = 92;
+		public const int INPLACE_BITWISE_AND = 93;
+		public const int INPLACE_SHIFT_LEFT = 94;
+		public const int INPLACE_SHIFT_RIGHT = 95;
+		public const int CMP_OPERATOR = 96;
+		public const int GREATER_THAN = 97;
+		public const int LESS_THAN = 98;
+		public const int ADD = 99;
+		public const int EXCLUSIVE_OR = 100;
+		public const int DIVISION = 101;
+		public const int MODULUS = 102;
+		public const int BITWISE_AND = 103;
+		public const int SHIFT_LEFT = 104;
+		public const int SHIFT_RIGHT = 105;
+		public const int EXPONENTIATION = 106;
+		public const int INCREMENT = 107;
+		public const int DECREMENT = 108;
+		public const int ONES_COMPLEMENT = 109;
+		public const int INT = 110;
+		public const int LONG = 111;
+		public const int RE_LITERAL = 112;
+		public const int DOUBLE = 113;
+		public const int FLOAT = 114;
+		public const int TIMESPAN = 115;
+		public const int LINE_CONTINUATION = 116;
+		public const int SL_COMMENT = 117;
+		public const int ML_COMMENT = 118;
+		public const int WS = 119;
+		public const int X_RE_LITERAL = 120;
+		public const int ESCAPED_EXPRESSION = 121;
+		public const int DQS_ESC = 122;
+		public const int SQS_ESC = 123;
+		public const int SESC = 124;
+		public const int RE_CHAR = 125;
+		public const int X_RE_CHAR = 126;
+		public const int RE_ESC = 127;
+		public const int DIGIT_GROUP = 128;
+		public const int REVERSE_DIGIT_GROUP = 129;
+		public const int ID_PREFIX = 130;
+		public const int ID_LETTER = 131;
+		public const int DIGIT = 132;
+		public const int HEXDIGIT = 133;
 		
 		
 	protected int _skipWhitespaceRegion = 0;
@@ -277,10 +279,11 @@ using Boo.Lang.Parser.Util;
 			literals.Add("public", 47);
 			literals.Add("namespace", 46);
 			literals.Add("break", 10);
-			literals.Add("while", 65);
+			literals.Add("while", 66);
 			literals.Add("end", 22);
+			literals.Add("then", 58);
 			literals.Add("raise", 50);
-			literals.Add("typeof", 61);
+			literals.Add("typeof", 62);
 			literals.Add("and", 8);
 			literals.Add("failure", 27);
 			literals.Add("not", 41);
@@ -294,23 +297,23 @@ using Boo.Lang.Parser.Util;
 			literals.Add("do", 19);
 			literals.Add("except", 26);
 			literals.Add("event", 25);
-			literals.Add("unless", 62);
+			literals.Add("unless", 63);
 			literals.Add("super", 55);
 			literals.Add("set", 53);
-			literals.Add("transient", 59);
+			literals.Add("transient", 60);
 			literals.Add("constructor", 16);
 			literals.Add("interface", 35);
 			literals.Add("of", 43);
 			literals.Add("is", 37);
 			literals.Add("internal", 36);
 			literals.Add("final", 28);
-			literals.Add("yield", 66);
+			literals.Add("yield", 67);
 			literals.Add("or", 44);
 			literals.Add("destructor", 18);
 			literals.Add("if", 39);
 			literals.Add("override", 45);
 			literals.Add("as", 9);
-			literals.Add("try", 58);
+			literals.Add("try", 59);
 			literals.Add("goto", 33);
 			literals.Add("enum", 24);
 			literals.Add("isa", 38);
@@ -320,7 +323,7 @@ using Boo.Lang.Parser.Util;
 			literals.Add("false", 31);
 			literals.Add("static", 56);
 			literals.Add("abstract", 7);
-			literals.Add("partial", 64);
+			literals.Add("partial", 65);
 			literals.Add("callable", 12);
 			literals.Add("get", 32);
 			literals.Add("continue", 11);
@@ -332,8 +335,8 @@ using Boo.Lang.Parser.Util;
 			literals.Add("in", 40);
 			literals.Add("self", 54);
 			literals.Add("ensure", 23);
-			literals.Add("true", 60);
-			literals.Add("virtual", 63);
+			literals.Add("true", 61);
+			literals.Add("virtual", 64);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -491,6 +494,12 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
+						case '?':
+						{
+							mNULLABLE_SUFFIX(true);
+							theRetToken = returnToken_;
+							break;
+						}
 						default:
 							if ((cached_LA1=='<') && (cached_LA2=='<') && (LA(3)=='='))
 							{
@@ -601,7 +610,7 @@ tryAgain:
 		_ttype = ID;
 		
 		{
-			if ((cached_LA1=='?'||cached_LA1=='@'))
+			if ((cached_LA1=='@'))
 			{
 				mID_PREFIX(false);
 			}
@@ -626,11 +635,11 @@ tryAgain:
 				}
 				else
 				{
-					goto _loop553_breakloop;
+					goto _loop563_breakloop;
 				}
 				
 			}
-_loop553_breakloop:			;
+_loop563_breakloop:			;
 		}    // ( ... )*
 		_ttype = testLiteralsTable(_ttype);
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -646,23 +655,7 @@ _loop553_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = ID_PREFIX;
 		
-		switch ( cached_LA1 )
-		{
-		case '@':
-		{
-			match('@');
-			break;
-		}
-		case '?':
-		{
-			match('?');
-			break;
-		}
-		default:
-		{
-			throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
-		}
-		 }
+		match('@');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
 			_token = makeToken(_ttype);
@@ -767,7 +760,7 @@ _loop553_breakloop:			;
 		_ttype = NEWLINE;
 		
 		{ // ( ... )+
-			int _cnt686=0;
+			int _cnt696=0;
 			for (;;)
 			{
 				if ((cached_LA1=='\n'||cached_LA1=='\r') && (true) && (true))
@@ -809,12 +802,12 @@ _loop553_breakloop:			;
 				}
 				else
 				{
-					if (_cnt686 >= 1) { goto _loop686_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt696 >= 1) { goto _loop696_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt686++;
+				_cnt696++;
 			}
-_loop686_breakloop:			;
+_loop696_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -843,7 +836,7 @@ _loop686_breakloop:			;
 			{
 				match("0x");
 				{ // ( ... )+
-					int _cnt558=0;
+					int _cnt568=0;
 					for (;;)
 					{
 						if ((tokenSet_2_.member(cached_LA1)))
@@ -852,12 +845,12 @@ _loop686_breakloop:			;
 						}
 						else
 						{
-							if (_cnt558 >= 1) { goto _loop558_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt568 >= 1) { goto _loop568_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt558++;
+						_cnt568++;
 					}
-_loop558_breakloop:					;
+_loop568_breakloop:					;
 				}    // ( ... )+
 			}
 			{
@@ -1247,11 +1240,11 @@ _loop558_breakloop:					;
 				}
 				default:
 				{
-					goto _loop718_breakloop;
+					goto _loop728_breakloop;
 				}
 				 }
 			}
-_loop718_breakloop:			;
+_loop728_breakloop:			;
 		}    // ( ... )*
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1267,7 +1260,7 @@ _loop718_breakloop:			;
 		_ttype = REVERSE_DIGIT_GROUP;
 		
 		{ // ( ... )+
-			int _cnt722=0;
+			int _cnt732=0;
 			for (;;)
 			{
 				if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && ((cached_LA2 >= '0' && cached_LA2 <= '9')) && ((LA(3) >= '0' && LA(3) <= '9')))
@@ -1293,12 +1286,12 @@ _loop718_breakloop:			;
 				}
 				else
 				{
-					if (_cnt722 >= 1) { goto _loop722_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt732 >= 1) { goto _loop732_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt722++;
+				_cnt732++;
 			}
-_loop722_breakloop:			;
+_loop732_breakloop:			;
 		}    // ( ... )+
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
 		{
@@ -1623,11 +1616,11 @@ _loop722_breakloop:			;
 			EnterSkipWhitespaceRegion();
 		}
 		{
-			bool synPredMatched602 = false;
+			bool synPredMatched612 = false;
 			if (((cached_LA1=='a')))
 			{
-				int _m602 = mark();
-				synPredMatched602 = true;
+				int _m612 = mark();
+				synPredMatched612 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1636,12 +1629,12 @@ _loop722_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched602 = false;
+					synPredMatched612 = false;
 				}
-				rewind(_m602);
+				rewind(_m612);
 				inputState.guessing--;
 			}
-			if ( synPredMatched602 )
+			if ( synPredMatched612 )
 			{
 				match("assembly:");
 				if (0==inputState.guessing)
@@ -1903,11 +1896,11 @@ _loop722_breakloop:			;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = DIVISION;
 		
-		bool synPredMatched623 = false;
+		bool synPredMatched633 = false;
 		if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 		{
-			int _m623 = mark();
-			synPredMatched623 = true;
+			int _m633 = mark();
+			synPredMatched633 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1916,12 +1909,12 @@ _loop722_breakloop:			;
 			}
 			catch (RecognitionException)
 			{
-				synPredMatched623 = false;
+				synPredMatched633 = false;
 			}
-			rewind(_m623);
+			rewind(_m633);
 			inputState.guessing--;
 		}
-		if ( synPredMatched623 )
+		if ( synPredMatched633 )
 		{
 			mML_COMMENT(false);
 			if (0==inputState.guessing)
@@ -1930,11 +1923,11 @@ _loop722_breakloop:			;
 			}
 		}
 		else {
-			bool synPredMatched625 = false;
+			bool synPredMatched635 = false;
 			if (((cached_LA1=='/') && (tokenSet_3_.member(cached_LA2)) && (tokenSet_4_.member(LA(3)))))
 			{
-				int _m625 = mark();
-				synPredMatched625 = true;
+				int _m635 = mark();
+				synPredMatched635 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1943,12 +1936,12 @@ _loop722_breakloop:			;
 				}
 				catch (RecognitionException)
 				{
-					synPredMatched625 = false;
+					synPredMatched635 = false;
 				}
-				rewind(_m625);
+				rewind(_m635);
 				inputState.guessing--;
 			}
-			if ( synPredMatched625 )
+			if ( synPredMatched635 )
 			{
 				mRE_LITERAL(false);
 				if (0==inputState.guessing)
@@ -1976,11 +1969,11 @@ _loop722_breakloop:			;
 									}
 									else
 									{
-										goto _loop630_breakloop;
+										goto _loop640_breakloop;
 									}
 									
 								}
-_loop630_breakloop:								;
+_loop640_breakloop:								;
 							}    // ( ... )*
 							if (0==inputState.guessing)
 							{
@@ -2033,11 +2026,11 @@ _loop630_breakloop:								;
 					match('*');
 				}
 				else {
-					bool synPredMatched671 = false;
+					bool synPredMatched681 = false;
 					if (((cached_LA1=='/') && (cached_LA2=='*') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m671 = mark();
-						synPredMatched671 = true;
+						int _m681 = mark();
+						synPredMatched681 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2046,12 +2039,12 @@ _loop630_breakloop:								;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched671 = false;
+							synPredMatched681 = false;
 						}
-						rewind(_m671);
+						rewind(_m681);
 						inputState.guessing--;
 					}
-					if ( synPredMatched671 )
+					if ( synPredMatched681 )
 					{
 						mML_COMMENT(false);
 					}
@@ -2065,11 +2058,11 @@ _loop630_breakloop:								;
 					}
 					else
 					{
-						goto _loop673_breakloop;
+						goto _loop683_breakloop;
 					}
 					}
 				}
-_loop673_breakloop:				;
+_loop683_breakloop:				;
 			}    // ( ... )*
 			match("*/");
 			if (0==inputState.guessing)
@@ -2091,7 +2084,7 @@ _loop673_breakloop:				;
 		
 		match('/');
 		{ // ( ... )+
-			int _cnt704=0;
+			int _cnt714=0;
 			for (;;)
 			{
 				if ((tokenSet_3_.member(cached_LA1)))
@@ -2100,12 +2093,12 @@ _loop673_breakloop:				;
 				}
 				else
 				{
-					if (_cnt704 >= 1) { goto _loop704_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt714 >= 1) { goto _loop714_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt704++;
+				_cnt714++;
 			}
-_loop704_breakloop:			;
+_loop714_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -2324,12 +2317,12 @@ _loop704_breakloop:			;
 			for (;;)
 			{
 				// nongreedy exit test
-				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop651_breakloop;
-				bool synPredMatched647 = false;
+				if ((cached_LA1=='"') && (cached_LA2=='"') && (LA(3)=='"')) goto _loop661_breakloop;
+				bool synPredMatched657 = false;
 				if (((cached_LA1=='$') && (cached_LA2=='{') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 				{
-					int _m647 = mark();
-					synPredMatched647 = true;
+					int _m657 = mark();
+					synPredMatched657 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -2338,12 +2331,12 @@ _loop704_breakloop:			;
 					}
 					catch (RecognitionException)
 					{
-						synPredMatched647 = false;
+						synPredMatched657 = false;
 					}
-					rewind(_m647);
+					rewind(_m657);
 					inputState.guessing--;
 				}
-				if ( synPredMatched647 )
+				if ( synPredMatched657 )
 				{
 					if (0==inputState.guessing)
 					{
@@ -2355,11 +2348,11 @@ _loop704_breakloop:			;
 					mESCAPED_EXPRESSION(false);
 				}
 				else {
-					bool synPredMatched649 = false;
+					bool synPredMatched659 = false;
 					if (((cached_LA1=='\\') && (cached_LA2=='$') && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe'))))
 					{
-						int _m649 = mark();
-						synPredMatched649 = true;
+						int _m659 = mark();
+						synPredMatched659 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -2368,12 +2361,12 @@ _loop704_breakloop:			;
 						}
 						catch (RecognitionException)
 						{
-							synPredMatched649 = false;
+							synPredMatched659 = false;
 						}
-						rewind(_m649);
+						rewind(_m659);
 						inputState.guessing--;
 					}
-					if ( synPredMatched649 )
+					if ( synPredMatched659 )
 					{
 						_saveIndex = text.Length;
 						match('\\');
@@ -2390,11 +2383,11 @@ _loop704_breakloop:			;
 					}
 					else
 					{
-						goto _loop651_breakloop;
+						goto _loop661_breakloop;
 					}
 					}
 				}
-_loop651_breakloop:				;
+_loop661_breakloop:				;
 			}    // ( ... )*
 			_saveIndex = text.Length;
 			match("\"\"\"");
@@ -2462,11 +2455,11 @@ _loop651_breakloop:				;
 					{    // ( ... )*
 						for (;;)
 						{
-							bool synPredMatched657 = false;
+							bool synPredMatched667 = false;
 							if (((cached_LA1=='$') && (cached_LA2=='{') && (tokenSet_5_.member(LA(3)))))
 							{
-								int _m657 = mark();
-								synPredMatched657 = true;
+								int _m667 = mark();
+								synPredMatched667 = true;
 								inputState.guessing++;
 								try {
 									{
@@ -2475,12 +2468,12 @@ _loop651_breakloop:				;
 								}
 								catch (RecognitionException)
 								{
-									synPredMatched657 = false;
+									synPredMatched667 = false;
 								}
-								rewind(_m657);
+								rewind(_m667);
 								inputState.guessing--;
 							}
-							if ( synPredMatched657 )
+							if ( synPredMatched667 )
 							{
 								if (0==inputState.guessing)
 								{
@@ -2501,11 +2494,11 @@ _loop651_breakloop:				;
 							}
 							else
 							{
-								goto _loop659_breakloop;
+								goto _loop669_breakloop;
 							}
 							
 						}
-_loop659_breakloop:						;
+_loop669_breakloop:						;
 					}    // ( ... )*
 					_saveIndex = text.Length;
 					match('"');
@@ -2604,11 +2597,11 @@ _loop659_breakloop:						;
 				}
 				else
 				{
-					goto _loop663_breakloop;
+					goto _loop673_breakloop;
 				}
 				
 			}
-_loop663_breakloop:			;
+_loop673_breakloop:			;
 		}    // ( ... )*
 		_saveIndex = text.Length;
 		match('\'');
@@ -2676,11 +2669,11 @@ _loop663_breakloop:			;
 				}
 				else
 				{
-					goto _loop667_breakloop;
+					goto _loop677_breakloop;
 				}
 				
 			}
-_loop667_breakloop:			;
+_loop677_breakloop:			;
 		}    // ( ... )*
 		if (0==inputState.guessing)
 		{
@@ -2700,7 +2693,7 @@ _loop667_breakloop:			;
 		_ttype = WS;
 		
 		{ // ( ... )+
-			int _cnt676=0;
+			int _cnt686=0;
 			for (;;)
 			{
 				switch ( cached_LA1 )
@@ -2726,12 +2719,12 @@ _loop667_breakloop:			;
 				}
 				default:
 				{
-					if (_cnt676 >= 1) { goto _loop676_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt686 >= 1) { goto _loop686_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				break; }
-				_cnt676++;
+				_cnt686++;
 			}
-_loop676_breakloop:			;
+_loop686_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -2772,7 +2765,7 @@ _loop676_breakloop:			;
 		text.Length = _saveIndex;
 		match('/');
 		{ // ( ... )+
-			int _cnt680=0;
+			int _cnt690=0;
 			for (;;)
 			{
 				if ((tokenSet_9_.member(cached_LA1)))
@@ -2781,12 +2774,12 @@ _loop676_breakloop:			;
 				}
 				else
 				{
-					if (_cnt680 >= 1) { goto _loop680_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt690 >= 1) { goto _loop690_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt680++;
+				_cnt690++;
 			}
-_loop680_breakloop:			;
+_loop690_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (0==inputState.guessing)
@@ -3149,7 +3142,7 @@ _loop680_breakloop:			;
 			case '8':  case '9':
 			{
 				{ // ( ... )+
-					int _cnt712=0;
+					int _cnt722=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && (tokenSet_5_.member(cached_LA2)) && (true))
@@ -3158,12 +3151,12 @@ _loop680_breakloop:			;
 						}
 						else
 						{
-							if (_cnt712 >= 1) { goto _loop712_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt722 >= 1) { goto _loop722_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt712++;
+						_cnt722++;
 					}
-_loop712_breakloop:					;
+_loop722_breakloop:					;
 				}    // ( ... )+
 				break;
 			}
@@ -3276,11 +3269,25 @@ _loop712_breakloop:					;
 		returnToken_ = _token;
 	}
 	
+	public void mNULLABLE_SUFFIX(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = NULLABLE_SUFFIX;
+		
+		match('?');
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	
 	private static long[] mk_tokenSet_0_()
 	{
 		long[] data = new long[3072];
-		data[0]=-9223372036854775808L;
+		data[0]=0L;
 		data[1]=576460745995190271L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
