@@ -1022,6 +1022,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0153-1.boo");
 		}
 		
+		[Test]
+		public void BCE0153_2()
+		{
+			RunCompilerTestCase(@"BCE0153-2.boo");
+		}
+		
 		[Test]
 		public void BCE0154_1()
 		{
@@ -1046,6 +1052,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BCE0156-1.boo");
 		}
 		
+		[Test]
+		public void BCE0157_1()
+		{
+			RunCompilerTestCase(@"BCE0157-1.boo");
+		}
+		
 
 		override protected string GetRelativeTestCasesPath()
 		{
