@@ -67,6 +67,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"collections-2.boo");
 		}
 		
+		[Test]
+		public void enumerable_shorthand_1()
+		{
+			RunCompilerTestCase(@"enumerable-shorthand-1.boo");
+		}
+		
 		[Test]
 		public void enumerable_type_inference_1()
 		{
@@ -151,6 +157,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"generic-array-3.boo");
 		}
 		
+		[Test]
+		public void generic_field_1()
+		{
+			RunCompilerTestCase(@"generic-field-1.boo");
+		}
+		
 		[Test]
 		public void generic_method_1()
 		{
@@ -211,6 +223,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"generic-type-resolution-2.boo");
 		}
 		
+		[Test]
+		public void inheritance_1()
+		{
+			RunCompilerTestCase(@"inheritance-1.boo");
+		}
+		
 		[Test]
 		public void interface_1()
 		{
@@ -343,6 +361,36 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"name-resolution-1.boo");
 		}
 		
+		[Test]
+		public void nullable_1()
+		{
+			RunCompilerTestCase(@"nullable-1.boo");
+		}
+		
+		[Test]
+		public void nullable_2()
+		{
+			RunCompilerTestCase(@"nullable-2.boo");
+		}
+		
+		[Test]
+		public void nullable_3()
+		{
+			RunCompilerTestCase(@"nullable-3.boo");
+		}
+		
+		[Test]
+		public void nullable_4()
+		{
+			RunCompilerTestCase(@"nullable-4.boo");
+		}
+		
+		[Test]
+		public void nullable_5()
+		{
+			RunCompilerTestCase(@"nullable-5.boo");
+		}
+		
 		[Test]
 		public void override_1()
 		{
