@@ -13,6 +13,54 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"array_ldelem.boo");
 		}
 		
+		[Test]
+		public void BOO_1005_1()
+		{
+			RunCompilerTestCase(@"BOO-1005-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1005_2()
+		{
+			RunCompilerTestCase(@"BOO-1005-2.boo");
+		}
+		
+		[Test]
+		public void BOO_1008_1()
+		{
+			RunCompilerTestCase(@"BOO-1008-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1009_1()
+		{
+			RunCompilerTestCase(@"BOO-1009-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1013_1()
+		{
+			RunCompilerTestCase(@"BOO-1013-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1013_2()
+		{
+			RunCompilerTestCase(@"BOO-1013-2.boo");
+		}
+		
+		[Test]
+		public void BOO_1016_1()
+		{
+			RunCompilerTestCase(@"BOO-1016-1.boo");
+		}
+		
+		[Test]
+		public void BOO_1016_2()
+		{
+			RunCompilerTestCase(@"BOO-1016-2.boo");
+		}
+		
 		[Test]
 		public void BOO_104_1()
 		{
@@ -727,6 +775,30 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-770-1.boo");
 		}
 		
+		[Ignore(" Preference for generic not complete")][Test]
+		public void BOO_779_1()
+		{
+			RunCompilerTestCase(@"BOO-779-1.boo");
+		}
+		
+		[Ignore(" Preference for generic not complete")][Test]
+		public void BOO_779_2()
+		{
+			RunCompilerTestCase(@"BOO-779-2.boo");
+		}
+		
+		[Ignore(" Non-IEnumerable definitions of GetEnumerator() not yet supported")][Test]
+		public void BOO_779_3()
+		{
+			RunCompilerTestCase(@"BOO-779-3.boo");
+		}
+		
+		[Test]
+		public void BOO_779_4()
+		{
+			RunCompilerTestCase(@"BOO-779-4.boo");
+		}
+		
 		[Test]
 		public void BOO_792_1()
 		{
@@ -769,6 +841,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-809-4.boo");
 		}
 		
+		[Test]
+		public void BOO_813_1()
+		{
+			RunCompilerTestCase(@"BOO-813-1.boo");
+		}
+		
 		[Test]
 		public void BOO_826()
 		{
@@ -781,6 +859,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-831-1.boo");
 		}
 		
+		[Test]
+		public void BOO_835_1()
+		{
+			RunCompilerTestCase(@"BOO-835-1.boo");
+		}
+		
 		[Test]
 		public void BOO_844_1()
 		{
@@ -823,6 +907,12 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-862-2.boo");
 		}
 		
+		[Test]
+		public void BOO_864_1()
+		{
+			RunCompilerTestCase(@"BOO-864-1.boo");
+		}
+		
 		[Test]
 		public void BOO_865_1()
 		{
@@ -895,12 +985,90 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"BOO-960.boo");
 		}
 		
+		[Test]
+		public void BOO_973_1()
+		{
+			RunCompilerTestCase(@"BOO-973-1.boo");
+		}
+		
+		[Test]
+		public void BOO_974_1()
+		{
+			RunCompilerTestCase(@"BOO-974-1.boo");
+		}
+		
+		[Test]
+		public void BOO_975_1()
+		{
+			RunCompilerTestCase(@"BOO-975-1.boo");
+		}
+		
+		[Test]
+		public void BOO_977_1()
+		{
+			RunCompilerTestCase(@"BOO-977-1.boo");
+		}
+		
+		[Test]
+		public void BOO_979_1()
+		{
+			RunCompilerTestCase(@"BOO-979-1.boo");
+		}
+		
+		[Test]
+		public void BOO_979_2()
+		{
+			RunCompilerTestCase(@"BOO-979-2.boo");
+		}
+		
+		[Test]
+		public void BOO_982_1()
+		{
+			RunCompilerTestCase(@"BOO-982-1.boo");
+		}
+		
+		[Test]
+		public void BOO_983_1()
+		{
+			RunCompilerTestCase(@"BOO-983-1.boo");
+		}
+		
+		[Test]
+		public void BOO_983_2()
+		{
+			RunCompilerTestCase(@"BOO-983-2.boo");
+		}
+		
+		[Test]
+		public void BOO_986_1()
+		{
+			RunCompilerTestCase(@"BOO-986-1.boo");
+		}
+		
 		[Test]
 		public void BOO_99_1()
 		{
 			RunCompilerTestCase(@"BOO-99-1.boo");
 		}
 		
+		[Test]
+		public void BOO_992_1()
+		{
+			RunCompilerTestCase(@"BOO-992-1.boo");
+		}
+		
+		[Test]
+		public void BOO_992_2()
+		{
+			RunCompilerTestCase(@"BOO-992-2.boo");
+		}
+		
+		[Test]
+		public void BOO_999_1()
+		{
+			RunCompilerTestCase(@"BOO-999-1.boo");
+		}
+		
 		[Test]
 		public void for_re_Split()
 		{
