@@ -252,6 +252,84 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"for-array-2.boo");
 		}
 		
+		[Test]
+		public void for_or_1()
+		{
+			RunCompilerTestCase(@"for_or-1.boo");
+		}
+		
+		[Test]
+		public void for_or_2()
+		{
+			RunCompilerTestCase(@"for_or-2.boo");
+		}
+		
+		[Test]
+		public void for_or_3()
+		{
+			RunCompilerTestCase(@"for_or-3.boo");
+		}
+		
+		[Test]
+		public void for_or_4()
+		{
+			RunCompilerTestCase(@"for_or-4.boo");
+		}
+		
+		[Test]
+		public void for_or_then_1()
+		{
+			RunCompilerTestCase(@"for_or_then-1.boo");
+		}
+		
+		[Test]
+		public void for_or_then_2()
+		{
+			RunCompilerTestCase(@"for_or_then-2.boo");
+		}
+		
+		[Test]
+		public void for_or_then_3()
+		{
+			RunCompilerTestCase(@"for_or_then-3.boo");
+		}
+		
+		[Test]
+		public void for_or_then_4()
+		{
+			RunCompilerTestCase(@"for_or_then-4.boo");
+		}
+		
+		[Test]
+		public void for_or_then_5()
+		{
+			RunCompilerTestCase(@"for_or_then-5.boo");
+		}
+		
+		[Test]
+		public void for_then_1()
+		{
+			RunCompilerTestCase(@"for_then-1.boo");
+		}
+		
+		[Test]
+		public void for_then_2()
+		{
+			RunCompilerTestCase(@"for_then-2.boo");
+		}
+		
+		[Test]
+		public void for_then_3()
+		{
+			RunCompilerTestCase(@"for_then-3.boo");
+		}
+		
+		[Test]
+		public void for_then_4()
+		{
+			RunCompilerTestCase(@"for_then-4.boo");
+		}
+		
 		[Test]
 		public void goto_1()
 		{
@@ -450,6 +528,84 @@ namespace BooCompiler.Tests
 			RunCompilerTestCase(@"while-8.boo");
 		}
 		
+		[Test]
+		public void while_or_1()
+		{
+			RunCompilerTestCase(@"while_or-1.boo");
+		}
+		
+		[Test]
+		public void while_or_2()
+		{
+			RunCompilerTestCase(@"while_or-2.boo");
+		}
+		
+		[Test]
+		public void while_or_3()
+		{
+			RunCompilerTestCase(@"while_or-3.boo");
+		}
+		
+		[Test]
+		public void while_or_4()
+		{
+			RunCompilerTestCase(@"while_or-4.boo");
+		}
+		
+		[Test]
+		public void while_or_then_1()
+		{
+			RunCompilerTestCase(@"while_or_then-1.boo");
+		}
+		
+		[Test]
+		public void while_or_then_2()
+		{
+			RunCompilerTestCase(@"while_or_then-2.boo");
+		}
+		
+		[Test]
+		public void while_or_then_3()
+		{
+			RunCompilerTestCase(@"while_or_then-3.boo");
+		}
+		
+		[Test]
+		public void while_or_then_4()
+		{
+			RunCompilerTestCase(@"while_or_then-4.boo");
+		}
+		
+		[Test]
+		public void while_or_then_5()
+		{
+			RunCompilerTestCase(@"while_or_then-5.boo");
+		}
+		
+		[Test]
+		public void while_then_1()
+		{
+			RunCompilerTestCase(@"while_then-1.boo");
+		}
+		
+		[Test]
+		public void while_then_2()
+		{
+			RunCompilerTestCase(@"while_then-2.boo");
+		}
+		
+		[Test]
+		public void while_then_3()
+		{
+			RunCompilerTestCase(@"while_then-3.boo");
+		}
+		
+		[Test]
+		public void while_then_4()
+		{
+			RunCompilerTestCase(@"while_then-4.boo");
+		}
+		
 
 		override protected string GetRelativeTestCasesPath()
 		{
